package org.xwiki.cache;

import java.util.Properties;

public interface CacheFactory
{
    /**
     * This component's role, used when code needs to look it up.
     */
    public final static String ROLE = CacheFactory.class.getName();
    
    /**
     * Create and return a cache that could be configured to be clustered.
     * 
     * @param cacheName
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newCache(String cacheName) throws CacheException;

    /**
     * Create and return a cache that could be configured to be clustered with the given capacity.
     * 
     * @param cacheName
     * @param capacity the maximum number of objects the cache can contain.
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newCache(String cacheName, int capacity) throws CacheException;

    /**
     * Create and return a custom cache.
     * 
     * @param cacheName
     * @param props the cache properties.
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newCache(String cacheName, Properties props) throws CacheException;

    /**
     * Create and return a custom cache with capacity.
     * 
     * @param cacheName
     * @param props the cache properties.
     * @param capacity the maximum number of objects the cache can contain.
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newCache(String cacheName, Properties props, int capacity) throws CacheException;
}
