package org.xwiki.cache;

import java.util.Properties;

public interface LocalCacheFactory
{
    /**
     * This component's role, used when code needs to look it up.
     */
    public final static String ROLE = LocalCacheFactory.class.getName();
    
    /**
     * Create and return a new local only (never clustered) cache.
     * 
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newLocalCache() throws CacheException;

    /**
     * Create and return a new local only (never clustered) cache with given capacity.
     * 
     * @param capacity
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newLocalCache(int capacity) throws CacheException;

    /**
     * Create and return a new custom local cache.
     * 
     * @props the cache properties.
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newLocalCache(Properties props) throws CacheException;

    /**
     * Create and return a new custom local cache with capacity.
     * 
     * @props the cache properties.
     * @param capacity the maximum number of objects the cache can contain.
     * @return a new {@link Cache}.
     * @throws CacheException
     */
    Cache newLocalCache(Properties props, int capacity) throws CacheException;
}
