package org.xwiki.cache.jbosscache.internal;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Cache;
import org.jboss.cache.CacheFactory;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.config.Configuration.CacheMode;
import org.xwiki.cache.LocalCacheFactory;
import org.xwiki.cache.CacheException;
import org.xwiki.component.phase.Initializable;
import org.xwiki.component.phase.InitializationException;

public class JBossCacheLocalCacheFactory implements LocalCacheFactory, Initializable
{
    /**
     * This component's role hint, used when code needs to look it up.
     */
    public static final String ROLEHINT = "oscache";

    private static final Log LOG = LogFactory.getLog(JBossCacheLocalCacheFactory.class);

    private static final String LOCAL_PROPS_FILENAME = "jbosscache-local.xml";

    private CacheFactory<String, Object> factory;

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.component.phase.Initializable#initialize()
     */
    public void initialize() throws InitializationException
    {
        LOG.info("Initializing JBossCacheLocalCacheFactory");

        factory = new DefaultCacheFactory<String, Object>();

        LOG.info("Initialized JBossCacheLocalCacheFactory");
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache()
     */
    public org.xwiki.cache.Cache newLocalCache() throws CacheException
    {
        Cache<String, Object> cache = factory.createCache(LOCAL_PROPS_FILENAME);

        cache.getConfiguration().setCacheMode(CacheMode.LOCAL);

        return new JBossCacheCache(cache);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(int)
     */
    public org.xwiki.cache.Cache newLocalCache(int capacity) throws CacheException
    {
        Cache<String, Object> cache = factory.createCache(LOCAL_PROPS_FILENAME);

        cache.getConfiguration().setCacheMode(CacheMode.LOCAL);
        cache.getConfiguration().setReplQueueMaxElements(capacity);

        return new JBossCacheCache(cache);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(java.util.Properties)
     */
    public org.xwiki.cache.Cache newLocalCache(Properties props) throws CacheException
    {
        Cache<String, Object> cache = factory.createCache(LOCAL_PROPS_FILENAME);

        cache.getConfiguration().setCacheMode(CacheMode.LOCAL);

        return new JBossCacheCache(cache);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(java.util.Properties, int)
     */
    public org.xwiki.cache.Cache newLocalCache(Properties props, int capacity)
        throws CacheException
    {
        Cache<String, Object> cache = factory.createCache(LOCAL_PROPS_FILENAME);

        cache.getConfiguration().setCacheMode(CacheMode.LOCAL);
        cache.getConfiguration().setReplQueueMaxElements(capacity);

        return new JBossCacheCache(cache);
    }
}
