package org.xwiki.cache.oscache.internal;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xwiki.cache.CacheFactory;
import org.xwiki.cache.Cache;
import org.xwiki.cache.CacheException;
import org.xwiki.component.phase.Initializable;
import org.xwiki.component.phase.InitializationException;

public class OSCacheCacheFactory extends AbstractOSCacheCacheFactory implements CacheFactory,
    Initializable, Runnable
{
    /**
     * This component's role hint, used when code needs to look it up.
     */
    public static final String ROLEHINT = "oscache";

    private static final Log LOG = LogFactory.getLog(OSCacheCacheFactory.class);

    private static final String PROPS_FILENAME = "oscache.properties";

    private Properties cacheProperties;

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.component.phase.Initializable#initialize()
     */
    public void initialize() throws InitializationException
    {
        LOG.info("Initializing OSCacheCacheFactory");

        this.cacheProperties = loadProps(PROPS_FILENAME);
        Thread watcherThread = new Thread(this, "OSCacheService Cache Monitor");
        watcherThread.start();

        LOG.info("Initialized OSCacheCacheFactory");
    }

    public Cache newCache(String cacheName) throws CacheException
    {
        OSCacheCache cache = new OSCacheCache(cacheProperties);
        cache.setName(cacheName);
        initCache(cache);

        return cache;
    }

    public Cache newCache(String cacheName, int capacity) throws CacheException
    {
        OSCacheCache cache = new OSCacheCache(cacheProperties, capacity);
        cache.setName(cacheName);
        initCache(cache);

        return cache;
    }

    public Cache newCache(String cacheName, Properties props) throws CacheException
    {
        OSCacheCache cache = new OSCacheCache(mergeProperties(props, cacheProperties));
        cache.setName(cacheName);
        initCache(cache);

        return cache;
    }

    public Cache newCache(String cacheName, Properties props, int capacity)
        throws CacheException
    {
        OSCacheCache cache = new OSCacheCache(mergeProperties(props, cacheProperties), capacity);
        cache.setName(cacheName);
        initCache(cache);

        return cache;
    }
}
