package org.xwiki.cache.oscache.internal;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xwiki.cache.LocalCacheFactory;
import org.xwiki.cache.Cache;
import org.xwiki.cache.CacheException;
import org.xwiki.component.phase.Initializable;
import org.xwiki.component.phase.InitializationException;

public class OSCacheLocalCacheFactory extends AbstractOSCacheCacheFactory implements
    LocalCacheFactory, Initializable
{
    /**
     * This component's role hint, used when code needs to look it up.
     */
    public static final String ROLEHINT = "oscache";

    private static final Log LOG = LogFactory.getLog(OSCacheLocalCacheFactory.class);

    private static final String LOCAL_PROPS_FILENAME = "oscache-local.properties";

    private Properties localCacheProperties;

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.component.phase.Initializable#initialize()
     */
    public void initialize() throws InitializationException
    {
        LOG.info("Initializing OSCacheLocalCacheFactory");

        this.localCacheProperties = loadProps(LOCAL_PROPS_FILENAME);
        Thread watcherThread = new Thread(this, "OSCacheService Local Cache Monitor");
        watcherThread.start();

        LOG.info("Initialized OSCacheLocalCacheFactory");
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache()
     */
    public Cache newLocalCache() throws CacheException
    {
        return new OSCacheCache(localCacheProperties);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(int)
     */
    public Cache newLocalCache(int capacity) throws CacheException
    {
        return new OSCacheCache(localCacheProperties, capacity);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(java.util.Properties)
     */
    public Cache newLocalCache(Properties props) throws CacheException
    {
        OSCacheCache cache = new OSCacheCache(mergeProperties(props, localCacheProperties));
        initCache(cache);

        return cache;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.xwiki.cache.LocalCacheFactory#newLocalCache(java.util.Properties, int)
     */
    public Cache newLocalCache(Properties props, int capacity) throws CacheException
    {
        OSCacheCache cache =
            new OSCacheCache(mergeProperties(props, localCacheProperties), capacity);
        initCache(cache);

        return cache;
    }
}
